/*******************************************************************************
 * Copyright (c) 2006, 2013 Oracle and/or its affiliates. All rights reserved.
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0
 * which accompanies this distribution.
 * The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
 * and the Eclipse Distribution License is available at
 * http://www.eclipse.org/org/documents/edl-v10.php.
 *
 * Contributors:
 *     Oracle - initial API and implementation
 *
 ******************************************************************************/
package org.eclipse.persistence.tools.mapping.orm;

import org.eclipse.persistence.annotations.BatchFetchType;
import org.eclipse.persistence.tools.mapping.ExternalForm;
import org.eclipse.persistence.tools.utility.TextRange;

/**
 * This interface represents the batch fetch defined by the EclipseLink JPA spec that are available
 * for configuration via the EclipseLink ORM xml.
 * <p>
 * Provisional API: This interface is part of an interim API that is still under development and
 * expected to change significantly before reaching stability. It is available at this early stage
 * to solicit feedback from pioneering adopters on the understanding that any code that uses this
 * API will almost certainly be broken (repeatedly) as the API evolves.<p>
 *
 * @version 2.6
 */
@SuppressWarnings("nls")
public interface ExternalBatchFetch extends ExternalForm {

	/**
	 * The node name used to store and retrieve the element encapsulated by this external form.
	 */
	String BATCH_FETCH = "batch-fetch";

	/**
	 * The attribute name used to store and retrieve the size property.
	 */
	String SIZE = "size";

	/**
	 * The attribute name used to store and retrieve the type property.
	 */
	String TYPE = "type";

	/**
	 * Returns the size.
	 */
	int getSize();

	/**
	 * Returns the {@link TextRange} of the size value.
	 */
	TextRange getSizeTextRange();

	/**
	 * Returns the batch fetch type.
	 */
	BatchFetchType getType();

	/**
	 * Returns the {@link TextRange} of the batch fetch type.
	 */
	TextRange getTypeTextRange();

	/**
	 * Sets the size.
	 */
	void setSize(int size);

	/**
	 * Sets the batch fetch type.
	 */
	void setType(BatchFetchType type);
}